//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  fregistry.hpp
//! \brief Registration of CFunc's.

#ifndef FREGISTRY_H
#define FREGISTRY_H

#include "func.hpp"
#include "registry.hpp"

//  Low-level function types.
typedef void (*func_0_i)(int);
typedef void (*func_0_d)(double);
typedef void (*func_0_s)(string);

typedef int (*func_i_i)(int);
typedef int (*func_i_d)(double);
typedef int (*func_i_s)(string);
typedef double (*func_d_i)(int);
typedef double (*func_d_d)(double);
typedef double (*func_d_s)(string);
typedef void (*func_e_e)(double&, double&, double, double);
typedef string (*func_s_s)(string);

typedef int (*func_i_ii)(int, int);
typedef int (*func_i_id)(int, double);
typedef int (*func_i_is)(int, string);
typedef int (*func_i_di)(double, int);
typedef int (*func_i_dd)(double, double);
typedef int (*func_i_si)(string, int);
typedef double (*func_d_ii)(int, int);
typedef double (*func_d_id)(int, double);
typedef double (*func_d_di)(double, int);
typedef double (*func_d_dd)(double, double);
typedef double (*func_d_sd)(string, double);
typedef void (*func_e_dd)(double&, double&, double, double);
typedef void (*func_e_ee)(double&, double&, double, double, double, double);
typedef void (*func_e_se)(double&, double&, string, double, double);
typedef string (*func_s_si)(string, int);
typedef string (*func_s_ss)(string, string);

typedef int (*func_i_iii)(int, int, int);
typedef double (*func_d_ddd)(double, double, double);
typedef void (*func_e_eee)(double&, double&, double, double, double, double, double, double);


//! A singleton class, holding a collection of CFunc's.

class SFuncRegistry : public IRegistryTmp<CFunc>, public triv::ISingleton<SFuncRegistry>
{
private:
    virtual string type_name() const { return "operator and function"; }
    void register_fct_template(
        const CFuncMetadata& _m, const char* _outtype, const char* _intypes, funcPtr _f);

public:
    void display_functions() const;
    void display_operators() const;

    void register_fct_0_i(const CFuncMetadata& _m, func_0_i _f)
    {
        register_fct_template(_m, "0", "i", (funcPtr)_f);
    }

    void register_fct_0_d(const CFuncMetadata& _m, func_0_d _f)
    {
        register_fct_template(_m, "0", "d", (funcPtr)_f);
    }

    void register_fct_0_s(const CFuncMetadata& _m, func_0_s _f)
    {
        register_fct_template(_m, "0", "s", (funcPtr)_f);
    }

    void register_fct_i_i(const CFuncMetadata& _m, func_i_i _f)
    {
        register_fct_template(_m, "i", "i", (funcPtr)_f);
    }

    void register_fct_i_d(const CFuncMetadata& _m, func_i_d _f)
    {
        register_fct_template(_m, "i", "d", (funcPtr)_f);
    }

    void register_fct_i_s(const CFuncMetadata& _m, func_i_s _f)
    {
        register_fct_template(_m, "i", "s", (funcPtr)_f);
    }

    void register_fct_d_i(const CFuncMetadata& _m, func_d_i _f)
    {
        register_fct_template(_m, "d", "i", (funcPtr)_f);
    }

    void register_fct_d_d(const CFuncMetadata& _m, func_d_d _f)
    {
        register_fct_template(_m, "d", "d", (funcPtr)_f);
    }

    void register_fct_d_s(const CFuncMetadata& _m, func_d_s _f)
    {
        register_fct_template(_m, "d", "s", (funcPtr)_f);
    }

    void register_fct_e_e(const CFuncMetadata& _m, func_e_e _f)
    {
        register_fct_template(_m, "e", "e", (funcPtr)_f);
    }

    void register_fct_i_ii(const CFuncMetadata& _m, func_i_ii _f)
    {
        register_fct_template(_m, "i", "ii", (funcPtr)_f);
    }

    void register_fct_i_id(const CFuncMetadata& _m, func_i_id _f)
    {
        register_fct_template(_m, "i", "id", (funcPtr)_f);
    }

    void register_fct_i_is(const CFuncMetadata& _m, func_i_is _f)
    {
        register_fct_template(_m, "i", "is", (funcPtr)_f);
    }

    void register_fct_i_di(const CFuncMetadata& _m, func_i_di _f)
    {
        register_fct_template(_m, "i", "di", (funcPtr)_f);
    }

    void register_fct_i_dd(const CFuncMetadata& _m, func_i_dd _f)
    {
        register_fct_template(_m, "i", "dd", (funcPtr)_f);
    }

    void register_fct_i_si(const CFuncMetadata& _m, func_i_si _f)
    {
        register_fct_template(_m, "i", "si", (funcPtr)_f);
    }

    void register_fct_d_ii(const CFuncMetadata& _m, func_d_ii _f)
    {
        register_fct_template(_m, "d", "ii", (funcPtr)_f);
    }

    void register_fct_d_dd(const CFuncMetadata& _m, func_d_dd _f)
    {
        register_fct_template(_m, "d", "dd", (funcPtr)_f);
    }

    void register_fct_d_sd(const CFuncMetadata& _m, func_d_sd _f)
    {
        register_fct_template(_m, "d", "sd", (funcPtr)_f);
    }

    void register_fct_e_dd(const CFuncMetadata& _m, func_e_dd _f)
    {
        register_fct_template(_m, "e", "dd", (funcPtr)_f);
    }

    void register_fct_e_ee(const CFuncMetadata& _m, func_e_ee _f)
    {
        register_fct_template(_m, "e", "ee", (funcPtr)_f);
    }

    void register_fct_e_se(const CFuncMetadata& _m, func_e_se _f)
    {
        register_fct_template(_m, "e", "se", (funcPtr)_f);
    }

    void register_fct_s_si(const CFuncMetadata& _m, func_s_si _f)
    {
        register_fct_template(_m, "s", "si", (funcPtr)_f);
    }

    void register_fct_s_ss(const CFuncMetadata& _m, func_s_ss _f)
    {
        register_fct_template(_m, "s", "ss", (funcPtr)_f);
    }

    void register_fct_i_iii(const CFuncMetadata& _m, func_i_iii _f)
    {
        register_fct_template(_m, "i", "iii", (funcPtr)_f);
    }

    void register_fct_d_ddd(const CFuncMetadata& _m, func_d_ddd _f)
    {
        register_fct_template(_m, "d", "ddd", (funcPtr)_f);
    }

    void register_fct_e_eee(const CFuncMetadata& _m, func_e_eee _f)
    {
        register_fct_template(_m, "e", "eee", (funcPtr)_f);
    }
};

#endif // FREGISTRY_H
