//**************************************************************************************************
//*  FRIDA: fast reliable interactive data analysis
//*  (C) Joachim Wuttke 1990-, v2(C++) 2001-
//*  http://apps.jcns.fz-juelich.de/frida
//**************************************************************************************************

//! \file  dualplot.hpp
//! \brief Collection NPlot of plot frames CPlot.

#include "axis.hpp"


//! One plot frame.

class CPlot
{
public:
    int iPlot; //!< The index of this frame in NPloWin::Plots.
    CAxis X; //!< Limits, log flag &c for x axis.
    CAxis Y; //!< Limits, log flag &c for y axis.
    // For data plotting:
    int maxpoints; //!< Maximum # points to be plotted without reduction.
    bool with_errors; //!< Plot error bars?
    // For curve plotting:
    int equipoints; //!< Start curve plot with this # grid points.
    bool refine; //!< Refine curve plot when appropriate?

    CPlot(int _iPlot, bool _logx, bool _logy);

    void gp_write(const std::string& in);
    void clear_frame();
    void plot_frame(const std::string& xlabel, const std::string& ylabel);
    void add_spec(
        bool as_line, bool new_style, int style_no, const std::vector<double>& xp,
        const std::vector<double>& yp, const std::vector<double>& dyp,
        const std::vector<std::string>& z, const std::string& xco, const std::string& yco,
        const std::string& info);
    void show_specs();
    void doc_TxLine(const std::string& line);
    void doc_PtTxLine(const std::string& line, int num);
    void doc_CvTxLine(const std::string& line, int num);
    void write_postscript(
        const std::string& ps_outdir, const std::string& ps_head, const std::string& ps_dict);
    void set_aux(const std::string& cmd);
    std::string info() const;

private:
    // TODO: some of this could be made local static, if class instances
    // are created anew instead of calling clearFrame().
    int gp_fifo; //!< Pipe to Gnuplot.
    int gp_fno; //!< Number of Gnuplot input file.
    std::string gp_fnames; //!< List of currently plotted Gnuplot input file names.

    int ps_fnum; //!< Postscript output file number.
    int ps_snum; //!< Slice number in Postscript file.
    int ps_pnum; //!< Spectrum number, for setting pstyle.
    int ps_cnum; //!< Curve number, for setting cstyle.
    void
    ps_ticktack(const std::vector<double>& Tacks, int ntpt, const double* ticklim, const CAxis* A);
    std::vector<std::string> ps_accu; //!< Main Postscript cache.
    std::vector<std::string> ps_Doc; //!< Special Postscript cache for doc lines ?.
};
